/*
Software to interface to a NAND-flash chip connected to a FT2232H IC.
(C) 2012 Jeroen Domburg (jeroen AT spritesmods.com)

This program is free software: you can redistribute it and/or modify
t under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include "ftdicomms.h"

#define BUFSZ (1024*256)
//#define BUFSZ ((224*48)*2+48)
//#define BUFSZ (7186)
struct ftdi_context m_ftdi;
unsigned char ftdiBuff[BUFSZ];
int ftdiBuffPos;

//Error handling routine.
static int error(const char *err) {
	printf("Error at %s: %s\n", err, ftdi_get_error_string(&m_ftdi));
	exit(0); //Dirty. Disable to continue after errors.
	return 0;
}

static void ftdiSendByte(char b) {
	ftdiBuff[ftdiBuffPos++]=b;
	if (ftdiBuffPos==BUFSZ) {
		ftdiBuffPos=0;
		if (ftdi_write_data(&m_ftdi, ftdiBuff, BUFSZ)<0) error("writing cmd");
//		ftdi_usb_purge_rx_buffer(&m_ftdi);
//		printf("Buff\n");
	}
}

void ftdiFlush(void) {
//	ftdi_usb_purge_tx_buffer(&m_ftdi);
	ftdi_async_complete(&m_ftdi, 1);
	if (ftdi_write_data_set_chunksize(&m_ftdi, ftdiBuffPos)) error("bitmode");
	if (ftdi_write_data_async(&m_ftdi, ftdiBuff, ftdiBuffPos)<0) error("writing cmd");
//	printf("Flush %i\n", ftdiBuffPos);
	ftdiBuffPos=0;
}

//Write bytes to the ADBUS and ACBUS lines
void ftdiWriteWord(int w) {
	ftdiSendByte(WRITE_EXTENDED);
	ftdiSendByte(w>>8);
	ftdiSendByte(w&0xff);
	ftdiSendByte(w&0xff);
}

//Write a byte to ADBUS only
void ftdiWritePixels(int w) {
	ftdiSendByte(WRITE_SHORT);
	ftdiSendByte(w&0xff);
	ftdiSendByte(w&0xff);
}

//Try to find the ftdi chip and open it.
int ftdiOpen(int vid, int pid, int doslow) {
	unsigned char slow=DIS_DIV_5;
	if (doslow) slow=EN_DIV_5;
	//If vid/pid is zero, use default FT2232H vid/pid.
	if (vid==0) vid=0x0403;
	if (pid==0) pid=0x6010;
	//Open FTDI communications
	if (ftdi_init(&m_ftdi)<0) return error("init");
	if (ftdi_usb_open(&m_ftdi, vid, pid)<0) return error("open");
	if (ftdi_set_bitmode(&m_ftdi, 0, BITMODE_MCU)<0) error("bitmode");
	if (ftdi_write_data_set_chunksize(&m_ftdi, BUFSZ)) error("bitmode");
	ftdi_usb_purge_buffers(&m_ftdi);
	if (ftdi_write_data(&m_ftdi, &slow, 1)<0) return error("writing div5 cmd");
	ftdi_read_data(&m_ftdi, &slow, 1);
	if (ftdi_set_latency_timer(&m_ftdi, 1)<0) return error("setting latency");
	ftdiBuffPos=0;
	return 1;
}

//Destructor: Close everything.
void ftdiClose(void) {
	ftdi_usb_close(&m_ftdi);
	ftdi_deinit(&m_ftdi);
}
